import { isString } from "@thi.ng/checks/is-string";
import { assert } from "@thi.ng/errors/assert";
import { illegalArgs } from "@thi.ng/errors/illegal-arguments";
import { gensym } from "./idgen.js";
function sym(type, ...xs) {
  let id;
  let opts;
  let init;
  switch (xs.length) {
    case 0:
      if (!isString(type)) {
        init = type;
        type = init.type;
      }
      break;
    case 1:
      if (isString(xs[0])) {
        id = xs[0];
      } else if (xs[0].tag) {
        init = xs[0];
      } else {
        opts = xs[0];
      }
      break;
    case 2:
      if (isString(xs[0])) {
        [id, opts] = xs;
      } else {
        [opts, init] = xs;
      }
      break;
    case 3:
      [id, opts, init] = xs;
      break;
    default:
      illegalArgs();
  }
  return {
    tag: "sym",
    type,
    id: id || gensym(),
    opts: opts || {},
    init
  };
}
const constSym = (type, id, opts, init) => sym(type, id || gensym(), { const: true, ...opts }, init);
const arraySym = (type, id, opts = {}, init) => {
  if (init && opts.num == null) {
    opts.num = init.length;
  }
  assert(opts.num != null, "missing array length");
  init && assert(
    opts.num === init.length,
    `expected ${opts.num} items in array, but got ${init.length}`
  );
  const atype = type + "[]";
  return {
    tag: "sym",
    type: atype,
    id: id || gensym(),
    opts,
    init: init ? {
      tag: "array_init",
      type: atype,
      init
    } : void 0
  };
};
const input = (type, id, opts) => sym(type, id, { q: "in", type: "in", ...opts });
const output = (type, id, opts) => sym(type, id, { q: "out", type: "out", ...opts });
const uniform = (type, id, opts) => sym(type, id, { q: "in", type: "uni", ...opts });
export {
  arraySym,
  constSym,
  input,
  output,
  sym,
  uniform
};
